"""
Pydantic Models for CMDB - system/central_management

Runtime validation models for system/central_management configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class CentralManagementServerListServerTypeEnum(str, Enum):
    """Allowed values for server_type field in server-list."""
    UPDATE = "update"
    RATING = "rating"
    VPATCH_QUERY = "vpatch-query"
    IOT_COLLECT = "iot-collect"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class CentralManagementServerList(BaseModel):
    """
    Child table model for server-list.
    
    Additional severs that the FortiGate can use for updates (for AV, IPS, updates) and ratings (for web filter and antispam ratings) servers.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    server_type: list[CentralManagementServerListServerTypeEnum] = Field(description="FortiGuard service type.")    
    addr_type: Literal["ipv4", "ipv6", "fqdn"] | None = Field(default="ipv4", description="Indicate whether the FortiGate communicates with the override server using an IPv4 address, an IPv6 address or a FQDN.")    
    server_address: str = Field(default="0.0.0.0", description="IPv4 address of override server.")    
    server_address6: str = Field(default="::", description="IPv6 address of override server.")    
    fqdn: str = Field(max_length=255, description="FQDN address of override server.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class CentralManagementModel(BaseModel):
    """
    Pydantic model for system/central_management configuration.
    
    Configure central management.
    
    Validation Rules:        - mode: pattern=        - type_: pattern=        - fortigate_cloud_sso_default_profile: max_length=35 pattern=        - schedule_config_restore: pattern=        - schedule_script_restore: pattern=        - allow_push_configuration: pattern=        - allow_push_firmware: pattern=        - allow_remote_firmware_upgrade: pattern=        - allow_monitor: pattern=        - serial_number: pattern=        - fmg: pattern=        - fmg_source_ip: pattern=        - fmg_source_ip6: pattern=        - local_cert: max_length=35 pattern=        - ca_cert: pattern=        - vdom: max_length=31 pattern=        - server_list: pattern=        - fmg_update_port: pattern=        - fmg_update_http_header: pattern=        - include_default_servers: pattern=        - enc_algorithm: pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    mode: Literal["normal", "backup"] | None = Field(default="normal", description="Central management mode.")    
    type_: Literal["fortimanager", "fortiguard", "none"] | None = Field(default="fortiguard", serialization_alias="type", description="Central management type.")    
    fortigate_cloud_sso_default_profile: str | None = Field(max_length=35, default=None, description="Override access profile. Permission is set to read-only without a FortiGate Cloud Central Management license.")  # datasource: ['system.accprofile.name']    
    schedule_config_restore: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowing the central management server to restore the configuration of this FortiGate.")    
    schedule_script_restore: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowing the central management server to restore the scripts stored on this FortiGate.")    
    allow_push_configuration: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowing the central management server to push configuration changes to this FortiGate.")    
    allow_push_firmware: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowing the central management server to push firmware updates to this FortiGate.")    
    allow_remote_firmware_upgrade: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable remotely upgrading the firmware on this FortiGate from the central management server.")    
    allow_monitor: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable allowing the central management server to remotely monitor this FortiGate unit.")    
    serial_number: str | None = Field(default=None, description="Serial number.")    
    fmg: str | None = Field(default=None, description="IP address or FQDN of the FortiManager.")    
    fmg_source_ip: str | None = Field(default="0.0.0.0", description="IPv4 source address that this FortiGate uses when communicating with FortiManager.")    
    fmg_source_ip6: str | None = Field(default="::", description="IPv6 source address that this FortiGate uses when communicating with FortiManager.")    
    local_cert: str | None = Field(max_length=35, default=None, description="Certificate to be used by FGFM protocol.")  # datasource: ['certificate.local.name']    
    ca_cert: str | None = Field(default=None, description="CA certificate to be used by FGFM protocol.")  # datasource: ['certificate.ca.name']    
    vdom: str | None = Field(max_length=31, default="root", description="Virtual domain (VDOM) name to use when communicating with FortiManager.")  # datasource: ['system.vdom.name']    
    server_list: list[CentralManagementServerList] = Field(default_factory=list, description="Additional severs that the FortiGate can use for updates (for AV, IPS, updates) and ratings (for web filter and antispam ratings) servers.")    
    fmg_update_port: Literal["8890", "443"] | None = Field(default="8890", description="Port used to communicate with FortiManager that is acting as a FortiGuard update server.")    
    fmg_update_http_header: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable inclusion of HTTP header in update request.")    
    include_default_servers: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable inclusion of public FortiGuard servers in the override server list.")    
    enc_algorithm: Literal["default", "high", "low"] | None = Field(default="high", description="Encryption strength for communications between the FortiGate and central management.")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('fortigate_cloud_sso_default_profile')
    @classmethod
    def validate_fortigate_cloud_sso_default_profile(cls, v: Any) -> Any:
        """
        Validate fortigate_cloud_sso_default_profile field.
        
        Datasource: ['system.accprofile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('local_cert')
    @classmethod
    def validate_local_cert(cls, v: Any) -> Any:
        """
        Validate local_cert field.
        
        Datasource: ['certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ca_cert')
    @classmethod
    def validate_ca_cert(cls, v: Any) -> Any:
        """
        Validate ca_cert field.
        
        Datasource: ['certificate.ca.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('vdom')
    @classmethod
    def validate_vdom(cls, v: Any) -> Any:
        """
        Validate vdom field.
        
        Datasource: ['system.vdom.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "CentralManagementModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_fortigate_cloud_sso_default_profile_references(self, client: Any) -> list[str]:
        """
        Validate fortigate_cloud_sso_default_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/accprofile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = CentralManagementModel(
            ...     fortigate_cloud_sso_default_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_fortigate_cloud_sso_default_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.central_management.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "fortigate_cloud_sso_default_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.accprofile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Fortigate-Cloud-Sso-Default-Profile '{value}' not found in "
                "system/accprofile"
            )        
        return errors    
    async def validate_local_cert_references(self, client: Any) -> list[str]:
        """
        Validate local_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = CentralManagementModel(
            ...     local_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_local_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.central_management.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "local_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Local-Cert '{value}' not found in "
                "certificate/local"
            )        
        return errors    
    async def validate_ca_cert_references(self, client: Any) -> list[str]:
        """
        Validate ca_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/ca        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = CentralManagementModel(
            ...     ca_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ca_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.central_management.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ca_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.ca.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ca-Cert '{value}' not found in "
                "certificate/ca"
            )        
        return errors    
    async def validate_vdom_references(self, client: Any) -> list[str]:
        """
        Validate vdom references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/vdom        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = CentralManagementModel(
            ...     vdom="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vdom_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.central_management.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vdom", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.vdom.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vdom '{value}' not found in "
                "system/vdom"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = CentralManagementModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.central_management.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_fortigate_cloud_sso_default_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_local_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ca_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_vdom_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "CentralManagementModel",    "CentralManagementServerList",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.210527Z
# ============================================================================


"""
Pydantic Models for CMDB - report/layout

Runtime validation models for report/layout configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class LayoutBodyItemTypeEnum(str, Enum):
    """Allowed values for type_ field in body-item."""
    TEXT = "text"
    IMAGE = "image"
    CHART = "chart"
    MISC = "misc"

class LayoutBodyItemTextComponentEnum(str, Enum):
    """Allowed values for text_component field in body-item."""
    TEXT = "text"
    HEADING1 = "heading1"
    HEADING2 = "heading2"
    HEADING3 = "heading3"

class LayoutBodyItemMiscComponentEnum(str, Enum):
    """Allowed values for misc_component field in body-item."""
    HLINE = "hline"
    PAGE_BREAK = "page-break"
    COLUMN_BREAK = "column-break"
    SECTION_START = "section-start"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class LayoutPageHeaderHeaderItem(BaseModel):
    """
    Child table model for page.header.header-item.
    
    Configure report header item.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Report item ID.")    
    description: str | None = Field(max_length=63, default=None, description="Description.")    
    type_: Literal["text", "image"] | None = Field(default="text", serialization_alias="type", description="Report item type.")    
    style: str | None = Field(max_length=71, default=None, description="Report item style.")    
    content: str | None = Field(max_length=511, default=None, description="Report item text content.")    
    img_src: str | None = Field(max_length=127, default=None, description="Report item image file name.")
class LayoutPageHeader(BaseModel):
    """
    Child table model for page.header.
    
    Configure report page header.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    style: str | None = Field(max_length=71, default=None, description="Report header style.")    
    header_item: list[LayoutPageHeaderHeaderItem] = Field(default_factory=list, description="Configure report header item.")
class LayoutPageFooterFooterItem(BaseModel):
    """
    Child table model for page.footer.footer-item.
    
    Configure report footer item.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Report item ID.")    
    description: str | None = Field(max_length=63, default=None, description="Description.")    
    type_: Literal["text", "image"] | None = Field(default="text", serialization_alias="type", description="Report item type.")    
    style: str | None = Field(max_length=71, default=None, description="Report item style.")    
    content: str | None = Field(max_length=511, default=None, description="Report item text content.")    
    img_src: str | None = Field(max_length=127, default=None, description="Report item image file name.")
class LayoutPageFooter(BaseModel):
    """
    Child table model for page.footer.
    
    Configure report page footer.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    style: str | None = Field(max_length=71, default=None, description="Report footer style.")    
    footer_item: list[LayoutPageFooterFooterItem] = Field(default_factory=list, description="Configure report footer item.")
class LayoutPage(BaseModel):
    """
    Child table model for page.
    
    Configure report page.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    paper: Literal["a4", "letter"] | None = Field(default="a4", description="Report page paper.")    
    column_break_before: list[Literal["heading1", "heading2", "heading3"]] = Field(default_factory=list, description="Report page auto column break before heading.")    
    page_break_before: list[Literal["heading1", "heading2", "heading3"]] = Field(default_factory=list, description="Report page auto page break before heading.")    
    options: list[Literal["header-on-first-page", "footer-on-first-page"]] = Field(default_factory=list, description="Report page options.")    
    header: list[LayoutPageHeader] = Field(default_factory=list, description="Configure report page header.")    
    footer: list[LayoutPageFooter] = Field(default_factory=list, description="Configure report page footer.")
class LayoutBodyItemParameters(BaseModel):
    """
    Child table model for body-item.parameters.
    
    Parameters.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    name: str = Field(max_length=127, description="Field name that match field of parameters defined in dataset.")    
    value: str = Field(max_length=1023, description="Value to replace corresponding field of parameters defined in dataset.")
class LayoutBodyItem(BaseModel):
    """
    Child table model for body-item.
    
    Configure report body item.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Report item ID.")    
    description: str | None = Field(max_length=63, default=None, description="Description.")    
    type_: LayoutBodyItemTypeEnum | None = Field(default=LayoutBodyItemTypeEnum.TEXT, serialization_alias="type", description="Report item type.")    
    style: str | None = Field(max_length=71, default=None, description="Report item style.")    
    top_n: int | None = Field(ge=0, le=4294967295, default=0, description="Value of top.")    
    parameters: list[LayoutBodyItemParameters] = Field(default_factory=list, description="Parameters.")    
    text_component: LayoutBodyItemTextComponentEnum | None = Field(default=LayoutBodyItemTextComponentEnum.TEXT, description="Report item text component.")    
    content: str | None = Field(max_length=511, default=None, description="Report item text content.")    
    img_src: str | None = Field(max_length=127, default=None, description="Report item image file name.")    
    chart: str | None = Field(max_length=71, default=None, description="Report item chart name.")    
    chart_options: list[Literal["include-no-data", "hide-title", "show-caption"]] = Field(default_factory=list, description="Report chart options.")    
    misc_component: LayoutBodyItemMiscComponentEnum | None = Field(default=LayoutBodyItemMiscComponentEnum.HLINE, description="Report item miscellaneous component.")    
    title: str | None = Field(max_length=511, default=None, description="Report section title.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class LayoutOptionsEnum(str, Enum):
    """Allowed values for options field."""
    INCLUDE_TABLE_OF_CONTENT = "include-table-of-content"
    AUTO_NUMBERING_HEADING = "auto-numbering-heading"
    VIEW_CHART_AS_HEADING = "view-chart-as-heading"
    SHOW_HTML_NAVBAR_BEFORE_HEADING = "show-html-navbar-before-heading"
    DUMMY_OPTION = "dummy-option"

class LayoutDayEnum(str, Enum):
    """Allowed values for day field."""
    SUNDAY = "sunday"
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"


# ============================================================================
# Main Model
# ============================================================================

class LayoutModel(BaseModel):
    """
    Pydantic model for report/layout configuration.
    
    Report layout configuration.
    
    Validation Rules:        - name: max_length=35 pattern=        - title: max_length=127 pattern=        - subtitle: max_length=127 pattern=        - description: max_length=127 pattern=        - style_theme: max_length=35 pattern=        - options: pattern=        - format_: pattern=        - schedule_type: pattern=        - day: pattern=        - time: pattern=        - cutoff_option: pattern=        - cutoff_time: pattern=        - email_send: pattern=        - email_recipients: max_length=511 pattern=        - max_pdf_report: min=1 max=365 pattern=        - page: pattern=        - body_item: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Report layout name.")    
    title: str | None = Field(max_length=127, default=None, description="Report title.")    
    subtitle: str | None = Field(max_length=127, default=None, description="Report subtitle.")    
    description: str | None = Field(max_length=127, default=None, description="Description.")    
    style_theme: str = Field(max_length=35, description="Report style theme.")    
    options: list[LayoutOptionsEnum] = Field(default_factory=list, description="Report layout options.")    
    format_: list[Literal["pdf"]] = Field(default_factory=list, serialization_alias="format", description="Report format.")    
    schedule_type: Literal["demand", "daily", "weekly"] | None = Field(default="daily", description="Report schedule type.")    
    day: LayoutDayEnum | None = Field(default=LayoutDayEnum.SUNDAY, description="Schedule days of week to generate report.")    
    time: str | None = Field(default=None, description="Schedule time to generate report (format = hh:mm).")    
    cutoff_option: Literal["run-time", "custom"] | None = Field(default="run-time", description="Cutoff-option is either run-time or custom.")    
    cutoff_time: str | None = Field(default=None, description="Custom cutoff time to generate report (format = hh:mm).")    
    email_send: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable sending emails after reports are generated.")    
    email_recipients: str | None = Field(max_length=511, default=None, description="Email recipients for generated reports.")    
    max_pdf_report: int | None = Field(ge=1, le=365, default=31, description="Maximum number of PDF reports to keep at one time (oldest report is overwritten).")    
    page: list[LayoutPage] = Field(default_factory=list, description="Configure report page.")    
    body_item: list[LayoutBodyItem] = Field(default_factory=list, description="Configure report body item.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "LayoutModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "LayoutModel",    "LayoutPage",    "LayoutPage.Header",    "LayoutPage.Header.HeaderItem",    "LayoutPage.Footer",    "LayoutPage.Footer.FooterItem",    "LayoutBodyItem",    "LayoutBodyItem.Parameters",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.338161Z
# ============================================================================


"""
Pydantic Models for CMDB - switch_controller/qos/queue_policy

Runtime validation models for switch_controller/qos/queue_policy configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class QueuePolicyCosQueue(BaseModel):
    """
    Child table model for cos-queue.
    
    COS queue configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="Cos queue ID.")    
    description: str | None = Field(max_length=63, default=None, description="Description of the COS queue.")    
    min_rate: int | None = Field(ge=0, le=4294967295, default=0, description="Minimum rate (0 - 4294967295 kbps, 0 to disable).")    
    max_rate: int | None = Field(ge=0, le=4294967295, default=0, description="Maximum rate (0 - 4294967295 kbps, 0 to disable).")    
    min_rate_percent: int | None = Field(ge=0, le=4294967295, default=0, description="Minimum rate (% of link speed).")    
    max_rate_percent: int | None = Field(ge=0, le=4294967295, default=0, description="Maximum rate (% of link speed).")    
    drop_policy: Literal["taildrop", "weighted-random-early-detection"] | None = Field(default="taildrop", description="COS queue drop policy.")    
    ecn: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable ECN packet marking to drop eligible packets.")    
    weight: int | None = Field(ge=0, le=4294967295, default=1, description="Weight of weighted round robin scheduling.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class QueuePolicyModel(BaseModel):
    """
    Pydantic model for switch_controller/qos/queue_policy configuration.
    
    Configure FortiSwitch QoS egress queue policy.
    
    Validation Rules:        - name: max_length=63 pattern=        - schedule: pattern=        - rate_by: pattern=        - cos_queue: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=63, description="QoS policy name.")    
    schedule: Literal["strict", "round-robin", "weighted"] = Field(default="round-robin", description="COS queue scheduling.")    
    rate_by: Literal["kbps", "percent"] = Field(default="kbps", description="COS queue rate by kbps or percent.")    
    cos_queue: list[QueuePolicyCosQueue] = Field(default_factory=list, description="COS queue configuration.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "QueuePolicyModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "QueuePolicyModel",    "QueuePolicyCosQueue",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.054166Z
# ============================================================================


"""
Pydantic Models for CMDB - application/list

Runtime validation models for application/list configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class ListEntriesPopularityEnum(str, Enum):
    """Allowed values for popularity field in entries."""
    V_1 = "1"
    V_2 = "2"
    V_3 = "3"
    V_4 = "4"
    V_5 = "5"

class ListEntriesRateTrackEnum(str, Enum):
    """Allowed values for rate_track field in entries."""
    NONE = "none"
    SRC_IP = "src-ip"
    DEST_IP = "dest-ip"
    DHCP_CLIENT_MAC = "dhcp-client-mac"
    DNS_DOMAIN = "dns-domain"

class ListDefaultNetworkServicesServicesEnum(str, Enum):
    """Allowed values for services field in default-network-services."""
    HTTP = "http"
    SSH = "ssh"
    TELNET = "telnet"
    FTP = "ftp"
    DNS = "dns"
    SMTP = "smtp"
    POP3 = "pop3"
    IMAP = "imap"
    SNMP = "snmp"
    NNTP = "nntp"
    HTTPS = "https"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ListEntriesRisk(BaseModel):
    """
    Child table model for entries.risk.
    
    Risk, or impact, of allowing traffic from this application to occur (1 - 5; Low, Elevated, Medium, High, and Critical).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    level: int = Field(ge=0, le=4294967295, default=0, description="Risk, or impact, of allowing traffic from this application to occur (1 - 5; Low, Elevated, Medium, High, and Critical).")
class ListEntriesParametersMembers(BaseModel):
    """
    Child table model for entries.parameters.members.
    
    Parameter tuple members.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Parameter.")    
    name: str = Field(max_length=31, description="Parameter name.")    
    value: str = Field(max_length=199, description="Parameter value.")
class ListEntriesParameters(BaseModel):
    """
    Child table model for entries.parameters.
    
    Application parameters.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Parameter tuple ID.")    
    members: list[ListEntriesParametersMembers] = Field(default_factory=list, description="Parameter tuple members.")
class ListEntriesExclusion(BaseModel):
    """
    Child table model for entries.exclusion.
    
    ID of excluded applications.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Excluded application IDs.")
class ListEntriesCategory(BaseModel):
    """
    Child table model for entries.category.
    
    Category ID list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Application category ID.")
class ListEntriesApplication(BaseModel):
    """
    Child table model for entries.application.
    
    ID of allowed applications.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Application IDs.")
class ListEntries(BaseModel):
    """
    Child table model for entries.
    
    Application list entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Entry ID.")    
    risk: list[ListEntriesRisk] = Field(default_factory=list, description="Risk, or impact, of allowing traffic from this application to occur (1 - 5; Low, Elevated, Medium, High, and Critical).")    
    category: list[ListEntriesCategory] = Field(default_factory=list, description="Category ID list.")    
    application: list[ListEntriesApplication] = Field(default_factory=list, description="ID of allowed applications.")    
    protocols: list[str] = Field(default_factory=list, description="Application protocol filter.")    
    vendor: list[str] = Field(default_factory=list, description="Application vendor filter.")    
    technology: list[str] = Field(default_factory=list, description="Application technology filter.")    
    behavior: list[str] = Field(default_factory=list, description="Application behavior filter.")    
    popularity: list[ListEntriesPopularityEnum] = Field(default_factory=list, description="Application popularity filter (1 - 5, from least to most popular).")    
    exclusion: list[ListEntriesExclusion] = Field(default_factory=list, description="ID of excluded applications.")    
    parameters: list[ListEntriesParameters] = Field(default_factory=list, description="Application parameters.")    
    action: Literal["pass", "block", "reset"] | None = Field(default="block", description="Pass or block traffic, or reset connection for traffic from this application.")    
    log: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable logging for this application list.")    
    log_packet: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable packet logging.")    
    rate_count: int | None = Field(ge=0, le=65535, default=0, description="Count of the rate.")    
    rate_duration: int | None = Field(ge=1, le=65535, default=60, description="Duration (sec) of the rate.")    
    rate_mode: Literal["periodical", "continuous"] | None = Field(default="continuous", description="Rate limit mode.")    
    rate_track: ListEntriesRateTrackEnum | None = Field(default=ListEntriesRateTrackEnum.NONE, description="Track the packet protocol field.")    
    session_ttl: int | None = Field(ge=0, le=4294967295, default=0, description="Session TTL (0 = default).")    
    shaper: str | None = Field(max_length=35, default=None, description="Traffic shaper.")  # datasource: ['firewall.shaper.traffic-shaper.name']    
    shaper_reverse: str | None = Field(max_length=35, default=None, description="Reverse traffic shaper.")  # datasource: ['firewall.shaper.traffic-shaper.name']    
    per_ip_shaper: str | None = Field(max_length=35, default=None, description="Per-IP traffic shaper.")  # datasource: ['firewall.shaper.per-ip-shaper.name']    
    quarantine: Literal["none", "attacker"] | None = Field(default="none", description="Quarantine method.")    
    quarantine_expiry: str | None = Field(default="5m", description="Duration of quarantine. (Format ###d##h##m, minimum 1m, maximum 364d23h59m, default = 5m). Requires quarantine set to attacker.")    
    quarantine_log: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable quarantine logging.")
class ListDefaultNetworkServices(BaseModel):
    """
    Child table model for default-network-services.
    
    Default network service entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Entry ID.")    
    port: int = Field(ge=0, le=65535, default=0, description="Port number.")    
    services: list[ListDefaultNetworkServicesServicesEnum] = Field(default_factory=list, description="Network protocols.")    
    violation_action: Literal["pass", "monitor", "block"] | None = Field(default="block", description="Action for protocols not in the allowlist for selected port.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ListOptionsEnum(str, Enum):
    """Allowed values for options field."""
    ALLOW_DNS = "allow-dns"
    ALLOW_ICMP = "allow-icmp"
    ALLOW_HTTP = "allow-http"
    ALLOW_SSL = "allow-ssl"


# ============================================================================
# Main Model
# ============================================================================

class ListModel(BaseModel):
    """
    Pydantic model for application/list configuration.
    
    Configure application control lists.
    
    Validation Rules:        - name: max_length=47 pattern=        - comment: max_length=255 pattern=        - replacemsg_group: max_length=35 pattern=        - extended_log: pattern=        - other_application_action: pattern=        - app_replacemsg: pattern=        - other_application_log: pattern=        - enforce_default_app_port: pattern=        - force_inclusion_ssl_di_sigs: pattern=        - unknown_application_action: pattern=        - unknown_application_log: pattern=        - p2p_block_list: pattern=        - deep_app_inspection: pattern=        - options: pattern=        - entries: pattern=        - control_default_network_services: pattern=        - default_network_services: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=47, description="List name.")    
    comment: str | None = Field(max_length=255, default=None, description="Comments.")    
    replacemsg_group: str | None = Field(max_length=35, default=None, description="Replacement message group.")  # datasource: ['system.replacemsg-group.name']    
    extended_log: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable extended logging.")    
    other_application_action: Literal["pass", "block"] | None = Field(default="pass", description="Action for other applications.")    
    app_replacemsg: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable replacement messages for blocked applications.")    
    other_application_log: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging for other applications.")    
    enforce_default_app_port: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable default application port enforcement for allowed applications.")    
    force_inclusion_ssl_di_sigs: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable forced inclusion of SSL deep inspection signatures.")    
    unknown_application_action: Literal["pass", "block"] | None = Field(default="pass", description="Pass or block traffic from unknown applications.")    
    unknown_application_log: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging for unknown applications.")    
    p2p_block_list: list[Literal["skype", "edonkey", "bittorrent"]] = Field(default_factory=list, description="P2P applications to be block listed.")    
    deep_app_inspection: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable deep application inspection.")    
    options: list[ListOptionsEnum] = Field(default_factory=list, description="Basic application protocol signatures allowed by default.")    
    entries: list[ListEntries] = Field(default_factory=list, description="Application list entries.")    
    control_default_network_services: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable enforcement of protocols over selected ports.")    
    default_network_services: list[ListDefaultNetworkServices] = Field(default_factory=list, description="Default network service entries.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('replacemsg_group')
    @classmethod
    def validate_replacemsg_group(cls, v: Any) -> Any:
        """
        Validate replacemsg_group field.
        
        Datasource: ['system.replacemsg-group.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ListModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_replacemsg_group_references(self, client: Any) -> list[str]:
        """
        Validate replacemsg_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/replacemsg-group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ListModel(
            ...     replacemsg_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_replacemsg_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.application.list.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "replacemsg_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.replacemsg_group.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Replacemsg-Group '{value}' not found in "
                "system/replacemsg-group"
            )        
        return errors    
    async def validate_entries_references(self, client: Any) -> list[str]:
        """
        Validate entries references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/shaper/per-ip-shaper        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ListModel(
            ...     entries=[{"per-ip-shaper": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_entries_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.application.list.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "entries", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("per-ip-shaper")
            else:
                value = getattr(item, "per-ip-shaper", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.shaper.per_ip_shaper.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Entries '{value}' not found in "
                    "firewall/shaper/per-ip-shaper"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_replacemsg_group_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_entries_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ListModel",    "ListEntries",    "ListEntries.Risk",    "ListEntries.Category",    "ListEntries.Application",    "ListEntries.Exclusion",    "ListEntries.Parameters",    "ListEntries.Parameters.Members",    "ListDefaultNetworkServices",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.550120Z
# ============================================================================


"""
Pydantic Models for CMDB - wireless_controller/bonjour_profile

Runtime validation models for wireless_controller/bonjour_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class BonjourProfilePolicyListServicesEnum(str, Enum):
    """Allowed values for services field in policy-list."""
    ALL = "all"
    AIRPLAY = "airplay"
    AFP = "afp"
    BIT_TORRENT = "bit-torrent"
    FTP = "ftp"
    ICHAT = "ichat"
    ITUNES = "itunes"
    PRINTERS = "printers"
    SAMBA = "samba"
    SCANNERS = "scanners"
    SSH = "ssh"
    CHROMECAST = "chromecast"
    MIRACAST = "miracast"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class BonjourProfilePolicyList(BaseModel):
    """
    Child table model for policy-list.
    
    Bonjour policy list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    policy_id: int | None = Field(ge=1, le=65535, default=0, description="Policy ID.")    
    description: str | None = Field(max_length=63, default=None, description="Description.")    
    from_vlan: str | None = Field(max_length=63, default="0", description="VLAN ID from which the Bonjour service is advertised (0 - 4094, default = 0).")    
    to_vlan: str | None = Field(max_length=63, default="all", description="VLAN ID to which the Bonjour service is made available (0 - 4094, default = all).")    
    services: list[BonjourProfilePolicyListServicesEnum] = Field(default_factory=list, description="Bonjour services for the VLAN connecting to the Bonjour network.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class BonjourProfileModel(BaseModel):
    """
    Pydantic model for wireless_controller/bonjour_profile configuration.
    
    Configure Bonjour profiles. Bonjour is Apple's zero configuration networking protocol. Bonjour profiles allow APs and FortiAPs to connect to networks using Bonjour.
    
    Validation Rules:        - name: max_length=35 pattern=        - comment: max_length=63 pattern=        - micro_location: pattern=        - policy_list: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Bonjour profile name.")    
    comment: str | None = Field(max_length=63, default=None, description="Comment.")    
    micro_location: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Micro location for Bonjour profile (default = disable).")    
    policy_list: list[BonjourProfilePolicyList] = Field(default_factory=list, description="Bonjour policy list.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "BonjourProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "BonjourProfileModel",    "BonjourProfilePolicyList",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.064334Z
# ============================================================================


"""
Pydantic Models for CMDB - system/switch_interface

Runtime validation models for system/switch_interface configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SwitchInterfaceSpanSourcePort(BaseModel):
    """
    Child table model for span-source-port.
    
    Physical interface name. Port spanning echoes all traffic on the SPAN source ports to the SPAN destination port.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    interface_name: str = Field(max_length=79, description="Physical interface name.")  # datasource: ['system.interface.name']
class SwitchInterfaceMember(BaseModel):
    """
    Child table model for member.
    
    Names of the interfaces that belong to the virtual switch.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    interface_name: str = Field(max_length=79, description="Interface name.")  # datasource: ['system.interface.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SwitchInterfaceModel(BaseModel):
    """
    Pydantic model for system/switch_interface configuration.
    
    Configure software switch interfaces by grouping physical and WiFi interfaces.
    
    Validation Rules:        - name: max_length=15 pattern=        - vdom: max_length=31 pattern=        - span_dest_port: max_length=15 pattern=        - span_source_port: pattern=        - member: pattern=        - type_: pattern=        - intra_switch_policy: pattern=        - mac_ttl: min=300 max=8640000 pattern=        - span: pattern=        - span_direction: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=15, description="Interface name (name cannot be in use by any other interfaces, VLANs, or inter-VDOM links).")    
    vdom: str = Field(max_length=31, description="VDOM that the software switch belongs to.")  # datasource: ['system.vdom.name']    
    span_dest_port: str | None = Field(max_length=15, default=None, description="SPAN destination port name. All traffic on the SPAN source ports is echoed to the SPAN destination port.")  # datasource: ['system.interface.name']    
    span_source_port: list[SwitchInterfaceSpanSourcePort] = Field(default_factory=list, description="Physical interface name. Port spanning echoes all traffic on the SPAN source ports to the SPAN destination port.")    
    member: list[SwitchInterfaceMember] = Field(default_factory=list, description="Names of the interfaces that belong to the virtual switch.")    
    type_: Literal["switch", "hub"] | None = Field(default="switch", serialization_alias="type", description="Type of switch based on functionality: switch for normal functionality, or hub to duplicate packets to all port members.")    
    intra_switch_policy: Literal["implicit", "explicit"] | None = Field(default="implicit", description="Allow any traffic between switch interfaces or require firewall policies to allow traffic between switch interfaces.")    
    mac_ttl: int | None = Field(ge=300, le=8640000, default=300, description="Duration for which MAC addresses are held in the ARP table (300 - 8640000 sec, default = 300).")    
    span: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable port spanning. Port spanning echoes traffic received by the software switch to the span destination port.")    
    span_direction: Literal["rx", "tx", "both"] | None = Field(default="both", description="The direction in which the SPAN port operates, either: rx, tx, or both.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('vdom')
    @classmethod
    def validate_vdom(cls, v: Any) -> Any:
        """
        Validate vdom field.
        
        Datasource: ['system.vdom.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('span_dest_port')
    @classmethod
    def validate_span_dest_port(cls, v: Any) -> Any:
        """
        Validate span_dest_port field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SwitchInterfaceModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_vdom_references(self, client: Any) -> list[str]:
        """
        Validate vdom references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/vdom        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SwitchInterfaceModel(
            ...     vdom="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vdom_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.switch_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vdom", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.vdom.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vdom '{value}' not found in "
                "system/vdom"
            )        
        return errors    
    async def validate_span_dest_port_references(self, client: Any) -> list[str]:
        """
        Validate span_dest_port references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SwitchInterfaceModel(
            ...     span_dest_port="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_span_dest_port_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.switch_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "span_dest_port", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Span-Dest-Port '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_span_source_port_references(self, client: Any) -> list[str]:
        """
        Validate span_source_port references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SwitchInterfaceModel(
            ...     span_source_port=[{"interface-name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_span_source_port_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.switch_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "span_source_port", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("interface-name")
            else:
                value = getattr(item, "interface-name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Span-Source-Port '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_member_references(self, client: Any) -> list[str]:
        """
        Validate member references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SwitchInterfaceModel(
            ...     member=[{"interface-name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_member_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.switch_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "member", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("interface-name")
            else:
                value = getattr(item, "interface-name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Member '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_vdom_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_span_dest_port_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_span_source_port_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_member_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SwitchInterfaceModel",    "SwitchInterfaceSpanSourcePort",    "SwitchInterfaceMember",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.164214Z
# ============================================================================


"""Type stubs for FORTIVIEW category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .historical_statistics import HistoricalStatistics
    from .realtime_proxy_statistics import RealtimeProxyStatistics
    from .realtime_statistics import RealtimeStatistics
    from .session import Session

__all__ = [
    "HistoricalStatistics",
    "RealtimeProxyStatistics",
    "RealtimeStatistics",
    "Fortiview",
]


class Fortiview:
    """FORTIVIEW API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    session: Session
    historical_statistics: HistoricalStatistics
    realtime_proxy_statistics: RealtimeProxyStatistics
    realtime_statistics: RealtimeStatistics

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiview category with HTTP client."""
        ...



"""
FortiManager Proxy Module

Enables FortiOS API calls to be routed through FortiManager to managed devices.

Example:
    >>> from hfortix_fortios import FortiManagerProxy
    >>>
    >>> # Connect to FortiManager
    >>> fmg = FortiManagerProxy(
    ...     host="fortimanager.example.com",
    ...     username="admin",
    ...     password="password",
    ... )
    >>>
    >>> # Get a proxied FortiOS client for a specific device
    >>> fgt = fmg.proxy(adom="production", device="firewall-01")
    >>>
    >>> # Use the exact same FortiOS API!
    >>> addresses = fgt.api.cmdb.firewall.address.get()
    >>> for addr in addresses:
    ...     print(f"{addr.name}: {addr.subnet}")
"""

from .client import FortiManagerProxy, ProxiedFortiOS, ProxyHTTPClient, FMGSession
from .models import ProxyResponse, DeviceResult

# Alias for backward compatibility
FMGProxyClient = ProxyHTTPClient

__all__ = [
    "FortiManagerProxy",
    "ProxiedFortiOS",
    "ProxyHTTPClient",
    "FMGProxyClient",  # Alias
    "FMGSession",
    "ProxyResponse",
    "DeviceResult",
]



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_TYPE: Literal["static", "dynamic", "ddns"]
VALID_BODY_IP_VERSION: Literal["4", "6"]
VALID_BODY_IKE_VERSION: Literal["1", "2"]
VALID_BODY_AUTHMETHOD: Literal["psk", "signature"]
VALID_BODY_AUTHMETHOD_REMOTE: Literal["psk", "signature"]
VALID_BODY_MODE: Literal["aggressive", "main"]
VALID_BODY_PEERTYPE: Literal["any", "one", "dialup", "peer", "peergrp"]
VALID_BODY_MONITOR_HOLD_DOWN_TYPE: Literal["immediate", "delay", "time"]
VALID_BODY_MONITOR_HOLD_DOWN_WEEKDAY: Literal["everyday", "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"]
VALID_BODY_NET_DEVICE: Literal["enable", "disable"]
VALID_BODY_PASSIVE_MODE: Literal["enable", "disable"]
VALID_BODY_EXCHANGE_INTERFACE_IP: Literal["enable", "disable"]
VALID_BODY_AGGREGATE_MEMBER: Literal["enable", "disable"]
VALID_BODY_PACKET_REDISTRIBUTION: Literal["enable", "disable"]
VALID_BODY_PEER_EGRESS_SHAPING: Literal["enable", "disable"]
VALID_BODY_MODE_CFG: Literal["disable", "enable"]
VALID_BODY_MODE_CFG_ALLOW_CLIENT_SELECTOR: Literal["disable", "enable"]
VALID_BODY_ASSIGN_IP: Literal["disable", "enable"]
VALID_BODY_ASSIGN_IP_FROM: Literal["range", "usrgrp", "dhcp", "name"]
VALID_BODY_DNS_MODE: Literal["manual", "auto"]
VALID_BODY_UNITY_SUPPORT: Literal["disable", "enable"]
VALID_BODY_INCLUDE_LOCAL_LAN: Literal["disable", "enable"]
VALID_BODY_SAVE_PASSWORD: Literal["disable", "enable"]
VALID_BODY_CLIENT_AUTO_NEGOTIATE: Literal["disable", "enable"]
VALID_BODY_CLIENT_KEEP_ALIVE: Literal["disable", "enable"]
VALID_BODY_PROPOSAL: Literal["des-md5", "des-sha1", "des-sha256", "des-sha384", "des-sha512", "3des-md5", "3des-sha1", "3des-sha256", "3des-sha384", "3des-sha512", "aes128-md5", "aes128-sha1", "aes128-sha256", "aes128-sha384", "aes128-sha512", "aes128gcm-prfsha1", "aes128gcm-prfsha256", "aes128gcm-prfsha384", "aes128gcm-prfsha512", "aes192-md5", "aes192-sha1", "aes192-sha256", "aes192-sha384", "aes192-sha512", "aes256-md5", "aes256-sha1", "aes256-sha256", "aes256-sha384", "aes256-sha512", "aes256gcm-prfsha1", "aes256gcm-prfsha256", "aes256gcm-prfsha384", "aes256gcm-prfsha512", "chacha20poly1305-prfsha1", "chacha20poly1305-prfsha256", "chacha20poly1305-prfsha384", "chacha20poly1305-prfsha512", "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-md5", "aria192-sha1", "aria192-sha256", "aria192-sha384", "aria192-sha512", "aria256-md5", "aria256-sha1", "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-md5", "seed-sha1", "seed-sha256", "seed-sha384", "seed-sha512"]
VALID_BODY_ADD_ROUTE: Literal["disable", "enable"]
VALID_BODY_ADD_GW_ROUTE: Literal["enable", "disable"]
VALID_BODY_LOCALID_TYPE: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"]
VALID_BODY_AUTO_NEGOTIATE: Literal["enable", "disable"]
VALID_BODY_FRAGMENTATION: Literal["enable", "disable"]
VALID_BODY_IP_FRAGMENTATION: Literal["pre-encapsulation", "post-encapsulation"]
VALID_BODY_DPD: Literal["disable", "on-idle", "on-demand"]
VALID_BODY_NPU_OFFLOAD: Literal["enable", "disable"]
VALID_BODY_SEND_CERT_CHAIN: Literal["enable", "disable"]
VALID_BODY_DHGRP: Literal["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"]
VALID_BODY_ADDKE1: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE2: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE3: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE4: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE5: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE6: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE7: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_SUITE_B: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"]
VALID_BODY_EAP: Literal["enable", "disable"]
VALID_BODY_EAP_IDENTITY: Literal["use-id-payload", "send-request"]
VALID_BODY_EAP_CERT_AUTH: Literal["enable", "disable"]
VALID_BODY_ACCT_VERIFY: Literal["enable", "disable"]
VALID_BODY_PPK: Literal["disable", "allow", "require"]
VALID_BODY_WIZARD_TYPE: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"]
VALID_BODY_XAUTHTYPE: Literal["disable", "client", "pap", "chap", "auto"]
VALID_BODY_REAUTH: Literal["disable", "enable"]
VALID_BODY_GROUP_AUTHENTICATION: Literal["enable", "disable"]
VALID_BODY_MESH_SELECTOR_TYPE: Literal["disable", "subnet", "host"]
VALID_BODY_IDLE_TIMEOUT: Literal["enable", "disable"]
VALID_BODY_SHARED_IDLE_TIMEOUT: Literal["enable", "disable"]
VALID_BODY_HA_SYNC_ESP_SEQNO: Literal["enable", "disable"]
VALID_BODY_FGSP_SYNC: Literal["enable", "disable"]
VALID_BODY_INBOUND_DSCP_COPY: Literal["enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_SENDER: Literal["enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_RECEIVER: Literal["enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_FORWARDER: Literal["enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_PSK: Literal["enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_SHORTCUTS: Literal["independent", "dependent"]
VALID_BODY_AUTO_DISCOVERY_CROSSOVER: Literal["allow", "block"]
VALID_BODY_AUTO_DISCOVERY_DIALUP_PLACEHOLDER: Literal["disable", "enable"]
VALID_BODY_ENCAPSULATION: Literal["none", "gre", "vxlan", "vpn-id-ipip"]
VALID_BODY_ENCAPSULATION_ADDRESS: Literal["ike", "ipv4", "ipv6"]
VALID_BODY_NATTRAVERSAL: Literal["enable", "disable", "forced"]
VALID_BODY_ESN: Literal["require", "allow", "disable"]
VALID_BODY_CHILDLESS_IKE: Literal["enable", "disable"]
VALID_BODY_AZURE_AD_AUTOCONNECT: Literal["enable", "disable"]
VALID_BODY_CLIENT_RESUME: Literal["enable", "disable"]
VALID_BODY_REKEY: Literal["enable", "disable"]
VALID_BODY_DIGITAL_SIGNATURE_AUTH: Literal["enable", "disable"]
VALID_BODY_SIGNATURE_HASH_ALG: Literal["sha1", "sha2-256", "sha2-384", "sha2-512"]
VALID_BODY_RSA_SIGNATURE_FORMAT: Literal["pkcs1", "pss"]
VALID_BODY_RSA_SIGNATURE_HASH_OVERRIDE: Literal["enable", "disable"]
VALID_BODY_ENFORCE_UNIQUE_ID: Literal["disable", "keep-new", "keep-old"]
VALID_BODY_CERT_ID_VALIDATION: Literal["enable", "disable"]
VALID_BODY_FEC_EGRESS: Literal["enable", "disable"]
VALID_BODY_FEC_CODEC: Literal["rs", "xor"]
VALID_BODY_FEC_INGRESS: Literal["enable", "disable"]
VALID_BODY_NETWORK_OVERLAY: Literal["disable", "enable"]
VALID_BODY_DEV_ID_NOTIFICATION: Literal["disable", "enable"]
VALID_BODY_LOOPBACK_ASYMROUTE: Literal["enable", "disable"]
VALID_BODY_EXCHANGE_FGT_DEVICE_ID: Literal["enable", "disable"]
VALID_BODY_IPV6_AUTO_LINKLOCAL: Literal["enable", "disable"]
VALID_BODY_EMS_SN_CHECK: Literal["enable", "disable"]
VALID_BODY_CERT_TRUST_STORE: Literal["local", "ems"]
VALID_BODY_QKD: Literal["disable", "allow", "require"]
VALID_BODY_QKD_HYBRID: Literal["disable", "allow", "require"]
VALID_BODY_TRANSPORT: Literal["udp", "auto", "tcp"]
VALID_BODY_FORTINET_ESP: Literal["enable", "disable"]
VALID_BODY_REMOTE_GW_MATCH: Literal["any", "ipmask", "iprange", "geography", "ztna"]
VALID_BODY_REMOTE_GW6_MATCH: Literal["any", "ipprefix", "iprange", "geography"]
VALID_BODY_CERT_PEER_USERNAME_VALIDATION: Literal["none", "othername", "rfc822name", "cn"]
VALID_BODY_CERT_PEER_USERNAME_STRIP: Literal["disable", "enable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_TYPE",
    "VALID_BODY_IP_VERSION",
    "VALID_BODY_IKE_VERSION",
    "VALID_BODY_AUTHMETHOD",
    "VALID_BODY_AUTHMETHOD_REMOTE",
    "VALID_BODY_MODE",
    "VALID_BODY_PEERTYPE",
    "VALID_BODY_MONITOR_HOLD_DOWN_TYPE",
    "VALID_BODY_MONITOR_HOLD_DOWN_WEEKDAY",
    "VALID_BODY_NET_DEVICE",
    "VALID_BODY_PASSIVE_MODE",
    "VALID_BODY_EXCHANGE_INTERFACE_IP",
    "VALID_BODY_AGGREGATE_MEMBER",
    "VALID_BODY_PACKET_REDISTRIBUTION",
    "VALID_BODY_PEER_EGRESS_SHAPING",
    "VALID_BODY_MODE_CFG",
    "VALID_BODY_MODE_CFG_ALLOW_CLIENT_SELECTOR",
    "VALID_BODY_ASSIGN_IP",
    "VALID_BODY_ASSIGN_IP_FROM",
    "VALID_BODY_DNS_MODE",
    "VALID_BODY_UNITY_SUPPORT",
    "VALID_BODY_INCLUDE_LOCAL_LAN",
    "VALID_BODY_SAVE_PASSWORD",
    "VALID_BODY_CLIENT_AUTO_NEGOTIATE",
    "VALID_BODY_CLIENT_KEEP_ALIVE",
    "VALID_BODY_PROPOSAL",
    "VALID_BODY_ADD_ROUTE",
    "VALID_BODY_ADD_GW_ROUTE",
    "VALID_BODY_LOCALID_TYPE",
    "VALID_BODY_AUTO_NEGOTIATE",
    "VALID_BODY_FRAGMENTATION",
    "VALID_BODY_IP_FRAGMENTATION",
    "VALID_BODY_DPD",
    "VALID_BODY_NPU_OFFLOAD",
    "VALID_BODY_SEND_CERT_CHAIN",
    "VALID_BODY_DHGRP",
    "VALID_BODY_ADDKE1",
    "VALID_BODY_ADDKE2",
    "VALID_BODY_ADDKE3",
    "VALID_BODY_ADDKE4",
    "VALID_BODY_ADDKE5",
    "VALID_BODY_ADDKE6",
    "VALID_BODY_ADDKE7",
    "VALID_BODY_SUITE_B",
    "VALID_BODY_EAP",
    "VALID_BODY_EAP_IDENTITY",
    "VALID_BODY_EAP_CERT_AUTH",
    "VALID_BODY_ACCT_VERIFY",
    "VALID_BODY_PPK",
    "VALID_BODY_WIZARD_TYPE",
    "VALID_BODY_XAUTHTYPE",
    "VALID_BODY_REAUTH",
    "VALID_BODY_GROUP_AUTHENTICATION",
    "VALID_BODY_MESH_SELECTOR_TYPE",
    "VALID_BODY_IDLE_TIMEOUT",
    "VALID_BODY_SHARED_IDLE_TIMEOUT",
    "VALID_BODY_HA_SYNC_ESP_SEQNO",
    "VALID_BODY_FGSP_SYNC",
    "VALID_BODY_INBOUND_DSCP_COPY",
    "VALID_BODY_AUTO_DISCOVERY_SENDER",
    "VALID_BODY_AUTO_DISCOVERY_RECEIVER",
    "VALID_BODY_AUTO_DISCOVERY_FORWARDER",
    "VALID_BODY_AUTO_DISCOVERY_PSK",
    "VALID_BODY_AUTO_DISCOVERY_SHORTCUTS",
    "VALID_BODY_AUTO_DISCOVERY_CROSSOVER",
    "VALID_BODY_AUTO_DISCOVERY_DIALUP_PLACEHOLDER",
    "VALID_BODY_ENCAPSULATION",
    "VALID_BODY_ENCAPSULATION_ADDRESS",
    "VALID_BODY_NATTRAVERSAL",
    "VALID_BODY_ESN",
    "VALID_BODY_CHILDLESS_IKE",
    "VALID_BODY_AZURE_AD_AUTOCONNECT",
    "VALID_BODY_CLIENT_RESUME",
    "VALID_BODY_REKEY",
    "VALID_BODY_DIGITAL_SIGNATURE_AUTH",
    "VALID_BODY_SIGNATURE_HASH_ALG",
    "VALID_BODY_RSA_SIGNATURE_FORMAT",
    "VALID_BODY_RSA_SIGNATURE_HASH_OVERRIDE",
    "VALID_BODY_ENFORCE_UNIQUE_ID",
    "VALID_BODY_CERT_ID_VALIDATION",
    "VALID_BODY_FEC_EGRESS",
    "VALID_BODY_FEC_CODEC",
    "VALID_BODY_FEC_INGRESS",
    "VALID_BODY_NETWORK_OVERLAY",
    "VALID_BODY_DEV_ID_NOTIFICATION",
    "VALID_BODY_LOOPBACK_ASYMROUTE",
    "VALID_BODY_EXCHANGE_FGT_DEVICE_ID",
    "VALID_BODY_IPV6_AUTO_LINKLOCAL",
    "VALID_BODY_EMS_SN_CHECK",
    "VALID_BODY_CERT_TRUST_STORE",
    "VALID_BODY_QKD",
    "VALID_BODY_QKD_HYBRID",
    "VALID_BODY_TRANSPORT",
    "VALID_BODY_FORTINET_ESP",
    "VALID_BODY_REMOTE_GW_MATCH",
    "VALID_BODY_REMOTE_GW6_MATCH",
    "VALID_BODY_CERT_PEER_USERNAME_VALIDATION",
    "VALID_BODY_CERT_PEER_USERNAME_STRIP",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""Type stubs for IPS category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .anomaly import Anomaly
    from .hold_signatures import HoldSignatures
    from .metadata import Metadata
    from .rate_based import RateBased
    from .session import Session

__all__ = [
    "Anomaly",
    "HoldSignatures",
    "Metadata",
    "RateBased",
    "Ips",
]


class Ips:
    """IPS API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    session: Session
    anomaly: Anomaly
    hold_signatures: HoldSignatures
    metadata: Metadata
    rate_based: RateBased

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize ips category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - wireless_controller/qos_profile

Runtime validation models for wireless_controller/qos_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class QosProfileDscpWmmVo(BaseModel):
    """
    Child table model for dscp-wmm-vo.
    
    DSCP mapping for voice access (default = 48 56).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=63, default=0, serialization_alias="id", description="DSCP WMM mapping numbers (0 - 63).")
class QosProfileDscpWmmVi(BaseModel):
    """
    Child table model for dscp-wmm-vi.
    
    DSCP mapping for video access (default = 32 40).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=63, default=0, serialization_alias="id", description="DSCP WMM mapping numbers (0 - 63).")
class QosProfileDscpWmmBk(BaseModel):
    """
    Child table model for dscp-wmm-bk.
    
    DSCP mapping for background access (default = 8 16).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=63, default=0, serialization_alias="id", description="DSCP WMM mapping numbers (0 - 63).")
class QosProfileDscpWmmBe(BaseModel):
    """
    Child table model for dscp-wmm-be.
    
    DSCP mapping for best effort access (default = 0 24).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=63, default=0, serialization_alias="id", description="DSCP WMM mapping numbers (0 - 63).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class QosProfileModel(BaseModel):
    """
    Pydantic model for wireless_controller/qos_profile configuration.
    
    Configure WiFi quality of service (QoS) profiles.
    
    Validation Rules:        - name: max_length=35 pattern=        - comment: max_length=63 pattern=        - uplink: min=0 max=2097152 pattern=        - downlink: min=0 max=2097152 pattern=        - uplink_sta: min=0 max=2097152 pattern=        - downlink_sta: min=0 max=2097152 pattern=        - burst: pattern=        - wmm: pattern=        - wmm_uapsd: pattern=        - call_admission_control: pattern=        - call_capacity: min=0 max=60 pattern=        - bandwidth_admission_control: pattern=        - bandwidth_capacity: min=1 max=600000 pattern=        - dscp_wmm_mapping: pattern=        - dscp_wmm_vo: pattern=        - dscp_wmm_vi: pattern=        - dscp_wmm_be: pattern=        - dscp_wmm_bk: pattern=        - wmm_dscp_marking: pattern=        - wmm_vo_dscp: min=0 max=63 pattern=        - wmm_vi_dscp: min=0 max=63 pattern=        - wmm_be_dscp: min=0 max=63 pattern=        - wmm_bk_dscp: min=0 max=63 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="WiFi QoS profile name.")    
    comment: str | None = Field(max_length=63, default=None, description="Comment.")    
    uplink: int | None = Field(ge=0, le=2097152, default=0, description="Maximum uplink bandwidth for Virtual Access Points (VAPs) (0 - 2097152 Kbps, default = 0, 0 means no limit).")    
    downlink: int | None = Field(ge=0, le=2097152, default=0, description="Maximum downlink bandwidth for Virtual Access Points (VAPs) (0 - 2097152 Kbps, default = 0, 0 means no limit).")    
    uplink_sta: int | None = Field(ge=0, le=2097152, default=0, description="Maximum uplink bandwidth for clients (0 - 2097152 Kbps, default = 0, 0 means no limit).")    
    downlink_sta: int | None = Field(ge=0, le=2097152, default=0, description="Maximum downlink bandwidth for clients (0 - 2097152 Kbps, default = 0, 0 means no limit).")    
    burst: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable client rate burst.")    
    wmm: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable WiFi multi-media (WMM) control.")    
    wmm_uapsd: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable WMM Unscheduled Automatic Power Save Delivery (U-APSD) power save mode.")    
    call_admission_control: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WMM call admission control.")    
    call_capacity: int | None = Field(ge=0, le=60, default=10, description="Maximum number of Voice over WLAN (VoWLAN) phones allowed (0 - 60, default = 10).")    
    bandwidth_admission_control: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WMM bandwidth admission control.")    
    bandwidth_capacity: int | None = Field(ge=1, le=600000, default=2000, description="Maximum bandwidth capacity allowed (1 - 600000 Kbps, default = 2000).")    
    dscp_wmm_mapping: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Differentiated Services Code Point (DSCP) mapping.")    
    dscp_wmm_vo: list[QosProfileDscpWmmVo] = Field(default_factory=list, description="DSCP mapping for voice access (default = 48 56).")    
    dscp_wmm_vi: list[QosProfileDscpWmmVi] = Field(default_factory=list, description="DSCP mapping for video access (default = 32 40).")    
    dscp_wmm_be: list[QosProfileDscpWmmBe] = Field(default_factory=list, description="DSCP mapping for best effort access (default = 0 24).")    
    dscp_wmm_bk: list[QosProfileDscpWmmBk] = Field(default_factory=list, description="DSCP mapping for background access (default = 8 16).")    
    wmm_dscp_marking: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WMM Differentiated Services Code Point (DSCP) marking.")    
    wmm_vo_dscp: int | None = Field(ge=0, le=63, default=48, description="DSCP marking for voice access (default = 48).")    
    wmm_vi_dscp: int | None = Field(ge=0, le=63, default=32, description="DSCP marking for video access (default = 32).")    
    wmm_be_dscp: int | None = Field(ge=0, le=63, default=0, description="DSCP marking for best effort access (default = 0).")    
    wmm_bk_dscp: int | None = Field(ge=0, le=63, default=8, description="DSCP marking for background access (default = 8).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "QosProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "QosProfileModel",    "QosProfileDscpWmmVo",    "QosProfileDscpWmmVi",    "QosProfileDscpWmmBe",    "QosProfileDscpWmmBk",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.687251Z
# ============================================================================


"""Type stubs for SWITCH_CONTROLLER category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .detected_device import DetectedDevice
    from .known_nac_device_criteria_list import KnownNacDeviceCriteriaList
    from .matched_devices import MatchedDevices
    from .fsw_firmware import FswFirmware
    from .isl_lockdown import IslLockdown
    from .managed_switch import ManagedSwitch
    from .mclag_icl import MclagIcl
    from .nac_device import NacDevice
    from .recommendation import Recommendation

__all__ = [
    "DetectedDevice",
    "KnownNacDeviceCriteriaList",
    "MatchedDevices",
    "SwitchController",
]


class SwitchController:
    """SWITCH_CONTROLLER API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    fsw_firmware: FswFirmware
    isl_lockdown: IslLockdown
    managed_switch: ManagedSwitch
    mclag_icl: MclagIcl
    nac_device: NacDevice
    recommendation: Recommendation
    detected_device: DetectedDevice
    known_nac_device_criteria_list: KnownNacDeviceCriteriaList
    matched_devices: MatchedDevices

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize switch_controller category with HTTP client."""
        ...



"""
Payload building helpers for FortiOS API.

Converts Python-style parameters to FortiOS API payloads with:
- snake_case to kebab-case conversion
- Optional list normalization
- None value filtering
"""

from typing import Any

from hfortix_fortios._helpers.normalizers import normalize_to_name_list


def build_cmdb_payload(**params: Any) -> dict[str, Any]:
    """
    Build a CMDB payload dictionary from keyword arguments (API layer - no normalization).

    Converts Python snake_case parameter names to FortiOS kebab-case API keys
    and filters out None values. This is the base helper used by all CMDB API endpoints.

    Does NOT normalize list fields - caller is responsible for providing data
    in the correct FortiOS format (unless using a wrapper with normalization).

    Args:
        **params: All resource parameters (e.g., name=..., member=..., etc.)

    Returns:
        Dictionary with FortiOS API-compatible keys and non-None values
    """
    payload: dict[str, Any] = {}

    # Extract 'data' parameter if present
    # It should be merged, not added as a key
    data_dict = params.pop("data", None)

    for param_name, value in params.items():
        if value is None:
            continue

        # Convert snake_case to kebab-case for FortiOS API
        api_key = param_name.replace("_", "-")
        payload[api_key] = value

    # Merge 'data' dictionary into payload (override existing keys)
    if data_dict and isinstance(data_dict, dict):
        payload.update(data_dict)

    return payload


def build_cmdb_payload_normalized(
    normalize_fields: set[str] | None = None, **params: Any
) -> dict[str, Any]:
    """
    Build a CMDB payload with automatic normalization (convenience wrapper layer).

    Converts Python snake_case parameter names to FortiOS kebab-case API keys,
    filters out None values, AND normalizes specified list fields to FortiOS format.

    This is used by convenience wrappers to accept flexible inputs like strings
    or lists and automatically convert them to FortiOS [{'name': '...'}] format.

    Args:
        normalize_fields: Set of field names (snake_case) that should be normalized
                         to [{'name': '...'}] format. If None, common fields like
                         'member', 'interface', 'allowaccess' are normalized.
        **params: All resource parameters

    Returns:
        Dictionary with FortiOS API-compatible keys and normalized values
    """
    # Default fields that commonly need normalization across CMDB endpoints
    DEFAULT_NORMALIZE_FIELDS = {
        "member",  # address groups, service groups, user groups
        "interface",  # various config objects
        "allowaccess",  # system interfaces
        "srcintf",  # firewall policies, routes
        "dstintf",  # firewall policies, routes
        "srcaddr",  # firewall policies
        "dstaddr",  # firewall policies
        "service",  # firewall policies
        "users",  # various auth/policy objects
        "groups",  # various auth/policy objects
    }

    # Use provided fields or defaults
    fields_to_normalize = (
        normalize_fields
        if normalize_fields is not None
        else DEFAULT_NORMALIZE_FIELDS
    )

    payload: dict[str, Any] = {}

    # Extract 'data' parameter if present
    # It should be merged, not added as a key
    data_dict = params.pop("data", None)

    for param_name, value in params.items():
        if value is None:
            continue

        # Convert snake_case to kebab-case for FortiOS API
        api_key = param_name.replace("_", "-")

        # Normalize list parameters to FortiOS format if specified
        if param_name in fields_to_normalize:
            normalized = normalize_to_name_list(value)
            # Only add if normalization resulted in non-empty list
            if normalized:
                payload[api_key] = normalized
        else:
            payload[api_key] = value

    # Merge 'data' dictionary into payload (override existing keys)
    if data_dict and isinstance(data_dict, dict):
        payload.update(data_dict)

    return payload


def build_api_payload(
    normalize_fields: set[str] | None = None,
    auto_normalize: bool = True,
    **params: Any,
) -> dict[str, Any]:
    """
    Build a generic API payload with intelligent list normalization.

    Universal helper for all API types (cmdb, monitor, log, service).
    Automatically normalizes common list fields that use [{'name': '...'}] format.

    Args:
        normalize_fields: Explicit set of field names (snake_case) to normalize.
                         If provided, only these fields are normalized.
        auto_normalize: If True and normalize_fields is None, auto-detect and
                       normalize common list fields. Set False for raw passthrough.
        **params: All resource parameters

    Returns:
        Dictionary with FortiOS API-compatible keys and normalized values
    """
    # Common list fields across all API types that use [{'name': '...'}] format
    COMMON_LIST_FIELDS = {
        # Firewall policy fields
        "srcintf",
        "dstintf",
        "srcaddr",
        "dstaddr",
        "srcaddr6",
        "dstaddr6",
        "service",
        "poolname",
        "poolname6",
        "groups",
        "users",
        "fsso_groups",
        "ztna_ems_tag",
        "ztna_ems_tag_secondary",
        "ztna_geo_tag",
        "internet_service_name",
        "internet_service_group",
        "internet_service_custom",
        "internet_service_custom_group",
        "network_service_dynamic",
        "internet_service_src_name",
        "internet_service_src_group",
        "internet_service_src_custom",
        "internet_service_src_custom_group",
        "network_service_src_dynamic",
        "internet_service6_name",
        "internet_service6_group",
        "internet_service6_custom",
        "internet_service6_custom_group",
        "internet_service6_src_name",
        "internet_service6_src_group",
        "internet_service6_src_custom",
        "internet_service6_src_custom_group",
        "src_vendor_mac",
        "rtp_addr",
        "ntlm_enabled_browsers",
        "custom_log_fields",
        "pcp_poolname",
        "sgt",
        "internet_service_fortiguard",
        "internet_service_src_fortiguard",
        "internet_service6_fortiguard",
        "internet_service6_src_fortiguard",
        # Group membership
        "member",
        # System/interface fields
        "interface",
        "allowaccess",
        "device",
        # Router fields
        "gateway",
        "nexthop",
        # VPN fields
        "destination",
        "source",
        # Application fields
        "application",
        "category",
        # User fields
        "group",
        "user",
        # Certificate fields
        "ca",
        "certificate",
        # DNS fields
        "dns_server",
    }

    # Determine which fields to normalize
    if normalize_fields is not None:
        # Explicit field list provided
        fields_to_normalize = normalize_fields
    elif auto_normalize:
        # Auto-detect common fields
        fields_to_normalize = COMMON_LIST_FIELDS
    else:
        # No normalization
        fields_to_normalize = set()

    payload: dict[str, Any] = {}

    # Extract 'data' parameter if present
    data_dict = params.pop("data", None)

    for param_name, value in params.items():
        if value is None:
            continue

        # Convert snake_case to kebab-case for FortiOS API
        api_key = param_name.replace("_", "-")

        # Normalize list parameters if specified
        if param_name in fields_to_normalize:
            normalized = normalize_to_name_list(value)
            # Only add if normalization resulted in non-empty list
            if normalized:
                payload[api_key] = normalized
        else:
            payload[api_key] = value

    # Merge 'data' dictionary into payload (override existing keys)
    if data_dict and isinstance(data_dict, dict):
        payload.update(data_dict)

    return payload



"""Type stubs for LICENSE category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .fortianalyzer_status import FortianalyzerStatus
    from .forticare_org_list import ForticareOrgList
    from .forticare_resellers import ForticareResellers
    from .status import Status
    from .database import Database

__all__ = [
    "FortianalyzerStatus",
    "ForticareOrgList",
    "ForticareResellers",
    "Status",
    "License",
]


class License:
    """LICENSE API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    database: Database
    fortianalyzer_status: FortianalyzerStatus
    forticare_org_list: ForticareOrgList
    forticare_resellers: ForticareResellers
    status: Status

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize license category with HTTP client."""
        ...



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_HTTPS_IMAGE_PUSH: Literal["enable", "disable"]
VALID_BODY_VLAN_ALL_MODE: Literal["all", "defined"]
VALID_BODY_VLAN_OPTIMIZATION: Literal["prune", "configured", "none"]
VALID_BODY_VLAN_IDENTITY: Literal["description", "name"]
VALID_BODY_DHCP_SERVER_ACCESS_LIST: Literal["enable", "disable"]
VALID_BODY_DHCP_OPTION82_FORMAT: Literal["ascii", "legacy"]
VALID_BODY_DHCP_OPTION82_CIRCUIT_ID: Literal["intfname", "vlan", "hostname", "mode", "description"]
VALID_BODY_DHCP_OPTION82_REMOTE_ID: Literal["mac", "hostname", "ip"]
VALID_BODY_DHCP_SNOOP_CLIENT_REQ: Literal["drop-untrusted", "forward-untrusted"]
VALID_BODY_LOG_MAC_LIMIT_VIOLATIONS: Literal["enable", "disable"]
VALID_BODY_SN_DNS_RESOLUTION: Literal["enable", "disable"]
VALID_BODY_MAC_EVENT_LOGGING: Literal["enable", "disable"]
VALID_BODY_BOUNCE_QUARANTINED_LINK: Literal["disable", "enable"]
VALID_BODY_QUARANTINE_MODE: Literal["by-vlan", "by-redirect"]
VALID_BODY_UPDATE_USER_DEVICE: Literal["mac-cache", "lldp", "dhcp-snooping", "l2-db", "l3-db"]
VALID_BODY_FIPS_ENFORCE: Literal["disable", "enable"]
VALID_BODY_FIRMWARE_PROVISION_ON_AUTHORIZATION: Literal["enable", "disable"]
VALID_BODY_SWITCH_ON_DEAUTH: Literal["no-op", "factory-reset"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_HTTPS_IMAGE_PUSH",
    "VALID_BODY_VLAN_ALL_MODE",
    "VALID_BODY_VLAN_OPTIMIZATION",
    "VALID_BODY_VLAN_IDENTITY",
    "VALID_BODY_DHCP_SERVER_ACCESS_LIST",
    "VALID_BODY_DHCP_OPTION82_FORMAT",
    "VALID_BODY_DHCP_OPTION82_CIRCUIT_ID",
    "VALID_BODY_DHCP_OPTION82_REMOTE_ID",
    "VALID_BODY_DHCP_SNOOP_CLIENT_REQ",
    "VALID_BODY_LOG_MAC_LIMIT_VIOLATIONS",
    "VALID_BODY_SN_DNS_RESOLUTION",
    "VALID_BODY_MAC_EVENT_LOGGING",
    "VALID_BODY_BOUNCE_QUARANTINED_LINK",
    "VALID_BODY_QUARANTINE_MODE",
    "VALID_BODY_UPDATE_USER_DEVICE",
    "VALID_BODY_FIPS_ENFORCE",
    "VALID_BODY_FIRMWARE_PROVISION_ON_AUTHORIZATION",
    "VALID_BODY_SWITCH_ON_DEAUTH",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""Type stubs for IPAM category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .list import List
    from .status import Status
    from .utilization import Utilization

__all__ = [
    "List",
    "Status",
    "Utilization",
    "Ipam",
]


class Ipam:
    """IPAM API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    list: List
    status: Status
    utilization: Utilization

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize ipam category with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/timers
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class TimersPayload(TypedDict, total=False):
    """Payload type for Timers operations."""
    echo_interval: int
    nat_session_keep_alive: int
    discovery_interval: int
    client_idle_timeout: int
    client_idle_rehome_timeout: int
    auth_timeout: int
    rogue_ap_log: int
    fake_ap_log: int
    sta_offline_cleanup: int
    sta_offline_ip2mac_cleanup: int
    sta_cap_cleanup: int
    rogue_ap_cleanup: int
    rogue_sta_cleanup: int
    wids_entry_cleanup: int
    ble_device_cleanup: int
    sta_stats_interval: int
    vap_stats_interval: int
    radio_stats_interval: int
    sta_capability_interval: int
    sta_locate_timer: int
    ipsec_intf_cleanup: int
    ble_scan_report_intv: int
    drma_interval: int
    ap_reboot_wait_interval1: int
    ap_reboot_wait_time: str
    ap_reboot_wait_interval2: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class TimersResponse(TypedDict, total=False):
    """Response type for Timers - use with .dict property for typed dict access."""
    echo_interval: int
    nat_session_keep_alive: int
    discovery_interval: int
    client_idle_timeout: int
    client_idle_rehome_timeout: int
    auth_timeout: int
    rogue_ap_log: int
    fake_ap_log: int
    sta_offline_cleanup: int
    sta_offline_ip2mac_cleanup: int
    sta_cap_cleanup: int
    rogue_ap_cleanup: int
    rogue_sta_cleanup: int
    wids_entry_cleanup: int
    ble_device_cleanup: int
    sta_stats_interval: int
    vap_stats_interval: int
    radio_stats_interval: int
    sta_capability_interval: int
    sta_locate_timer: int
    ipsec_intf_cleanup: int
    ble_scan_report_intv: int
    drma_interval: int
    ap_reboot_wait_interval1: int
    ap_reboot_wait_time: str
    ap_reboot_wait_interval2: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class TimersObject(FortiObject):
    """Typed FortiObject for Timers with field access."""
    echo_interval: int
    nat_session_keep_alive: int
    discovery_interval: int
    client_idle_timeout: int
    client_idle_rehome_timeout: int
    auth_timeout: int
    rogue_ap_log: int
    fake_ap_log: int
    sta_offline_cleanup: int
    sta_offline_ip2mac_cleanup: int
    sta_cap_cleanup: int
    rogue_ap_cleanup: int
    rogue_sta_cleanup: int
    wids_entry_cleanup: int
    ble_device_cleanup: int
    sta_stats_interval: int
    vap_stats_interval: int
    radio_stats_interval: int
    sta_capability_interval: int
    sta_locate_timer: int
    ipsec_intf_cleanup: int
    ble_scan_report_intv: int
    drma_interval: int
    ap_reboot_wait_interval1: int
    ap_reboot_wait_time: str
    ap_reboot_wait_interval2: int


# ================================================================
# Main Endpoint Class
# ================================================================

class Timers:
    """
    
    Endpoint: wireless_controller/timers
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> TimersObject: ...
    
    def get_schema(
        self,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: TimersPayload | None = ...,
        echo_interval: int | None = ...,
        nat_session_keep_alive: int | None = ...,
        discovery_interval: int | None = ...,
        client_idle_timeout: int | None = ...,
        client_idle_rehome_timeout: int | None = ...,
        auth_timeout: int | None = ...,
        rogue_ap_log: int | None = ...,
        fake_ap_log: int | None = ...,
        sta_offline_cleanup: int | None = ...,
        sta_offline_ip2mac_cleanup: int | None = ...,
        sta_cap_cleanup: int | None = ...,
        rogue_ap_cleanup: int | None = ...,
        rogue_sta_cleanup: int | None = ...,
        wids_entry_cleanup: int | None = ...,
        ble_device_cleanup: int | None = ...,
        sta_stats_interval: int | None = ...,
        vap_stats_interval: int | None = ...,
        radio_stats_interval: int | None = ...,
        sta_capability_interval: int | None = ...,
        sta_locate_timer: int | None = ...,
        ipsec_intf_cleanup: int | None = ...,
        ble_scan_report_intv: int | None = ...,
        drma_interval: int | None = ...,
        ap_reboot_wait_interval1: int | None = ...,
        ap_reboot_wait_time: str | None = ...,
        ap_reboot_wait_interval2: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> TimersObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: TimersPayload | None = ...,
        echo_interval: int | None = ...,
        nat_session_keep_alive: int | None = ...,
        discovery_interval: int | None = ...,
        client_idle_timeout: int | None = ...,
        client_idle_rehome_timeout: int | None = ...,
        auth_timeout: int | None = ...,
        rogue_ap_log: int | None = ...,
        fake_ap_log: int | None = ...,
        sta_offline_cleanup: int | None = ...,
        sta_offline_ip2mac_cleanup: int | None = ...,
        sta_cap_cleanup: int | None = ...,
        rogue_ap_cleanup: int | None = ...,
        rogue_sta_cleanup: int | None = ...,
        wids_entry_cleanup: int | None = ...,
        ble_device_cleanup: int | None = ...,
        sta_stats_interval: int | None = ...,
        vap_stats_interval: int | None = ...,
        radio_stats_interval: int | None = ...,
        sta_capability_interval: int | None = ...,
        sta_locate_timer: int | None = ...,
        ipsec_intf_cleanup: int | None = ...,
        ble_scan_report_intv: int | None = ...,
        drma_interval: int | None = ...,
        ap_reboot_wait_interval1: int | None = ...,
        ap_reboot_wait_time: str | None = ...,
        ap_reboot_wait_interval2: int | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Timers",
    "TimersPayload",
    "TimersResponse",
    "TimersObject",
]


"""FortiOS CMDB - Botnet category (stub)"""

from typing import Any
from ..botnet_base import Botnet as BotnetBase
from .stat import Stat

class Botnet(BotnetBase):
    """Botnet endpoints wrapper for CMDB API."""

    stat: Stat

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""
Pydantic Models for CMDB - authentication/setting

Runtime validation models for authentication/setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SettingUserCertCa(BaseModel):
    """
    Child table model for user-cert-ca.
    
    CA certificate used for client certificate verification.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="CA certificate list.")  # datasource: ['vpn.certificate.ca.name', 'vpn.certificate.local.name']
class SettingDevRange(BaseModel):
    """
    Child table model for dev-range.
    
    Address range for the IP based device query.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Address name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SettingModel(BaseModel):
    """
    Pydantic model for authentication/setting configuration.
    
    Configure authentication setting.
    
    Validation Rules:        - active_auth_scheme: max_length=35 pattern=        - sso_auth_scheme: max_length=35 pattern=        - update_time: pattern=        - persistent_cookie: pattern=        - ip_auth_cookie: pattern=        - cookie_max_age: min=30 max=10080 pattern=        - cookie_refresh_div: min=2 max=4 pattern=        - captive_portal_type: pattern=        - captive_portal_ip: pattern=        - captive_portal_ip6: pattern=        - captive_portal: max_length=255 pattern=        - captive_portal6: max_length=255 pattern=        - cert_auth: pattern=        - cert_captive_portal: max_length=255 pattern=        - cert_captive_portal_ip: pattern=        - cert_captive_portal_port: min=1 max=65535 pattern=        - captive_portal_port: min=1 max=65535 pattern=        - auth_https: pattern=        - captive_portal_ssl_port: min=1 max=65535 pattern=        - user_cert_ca: pattern=        - dev_range: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    active_auth_scheme: str | None = Field(max_length=35, default=None, description="Active authentication method (scheme name).")  # datasource: ['authentication.scheme.name']    
    sso_auth_scheme: str | None = Field(max_length=35, default=None, description="Single-Sign-On authentication method (scheme name).")  # datasource: ['authentication.scheme.name']    
    update_time: str | None = Field(default=None, description="Time of the last update.")    
    persistent_cookie: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable persistent cookie on web portal authentication (default = enable).")    
    ip_auth_cookie: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable persistent cookie on IP based web portal authentication (default = disable).")    
    cookie_max_age: int | None = Field(ge=30, le=10080, default=480, description="Persistent web portal cookie maximum age in minutes (30 - 10080 (1 week), default = 480 (8 hours)).")    
    cookie_refresh_div: int | None = Field(ge=2, le=4, default=2, description="Refresh rate divider of persistent web portal cookie (default = 2). Refresh value = cookie-max-age/cookie-refresh-div.")    
    captive_portal_type: Literal["fqdn", "ip"] | None = Field(default="fqdn", description="Captive portal type.")    
    captive_portal_ip: str | None = Field(default="0.0.0.0", description="Captive portal IP address.")    
    captive_portal_ip6: str | None = Field(default="::", description="Captive portal IPv6 address.")    
    captive_portal: str | None = Field(max_length=255, default=None, description="Captive portal host name.")  # datasource: ['firewall.address.name']    
    captive_portal6: str | None = Field(max_length=255, default=None, description="IPv6 captive portal host name.")  # datasource: ['firewall.address6.name']    
    cert_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable redirecting certificate authentication to HTTPS portal.")    
    cert_captive_portal: str | None = Field(max_length=255, default=None, description="Certificate captive portal host name.")  # datasource: ['firewall.address.name']    
    cert_captive_portal_ip: str | None = Field(default="0.0.0.0", description="Certificate captive portal IP address.")    
    cert_captive_portal_port: int | None = Field(ge=1, le=65535, default=7832, description="Certificate captive portal port number (1 - 65535, default = 7832).")    
    captive_portal_port: int | None = Field(ge=1, le=65535, default=7830, description="Captive portal port number (1 - 65535, default = 7830).")    
    auth_https: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable redirecting HTTP user authentication to HTTPS.")    
    captive_portal_ssl_port: int | None = Field(ge=1, le=65535, default=7831, description="Captive portal SSL port number (1 - 65535, default = 7831).")    
    user_cert_ca: list[SettingUserCertCa] = Field(default_factory=list, description="CA certificate used for client certificate verification.")    
    dev_range: list[SettingDevRange] = Field(default_factory=list, description="Address range for the IP based device query.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('active_auth_scheme')
    @classmethod
    def validate_active_auth_scheme(cls, v: Any) -> Any:
        """
        Validate active_auth_scheme field.
        
        Datasource: ['authentication.scheme.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('sso_auth_scheme')
    @classmethod
    def validate_sso_auth_scheme(cls, v: Any) -> Any:
        """
        Validate sso_auth_scheme field.
        
        Datasource: ['authentication.scheme.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('captive_portal')
    @classmethod
    def validate_captive_portal(cls, v: Any) -> Any:
        """
        Validate captive_portal field.
        
        Datasource: ['firewall.address.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('captive_portal6')
    @classmethod
    def validate_captive_portal6(cls, v: Any) -> Any:
        """
        Validate captive_portal6 field.
        
        Datasource: ['firewall.address6.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('cert_captive_portal')
    @classmethod
    def validate_cert_captive_portal(cls, v: Any) -> Any:
        """
        Validate cert_captive_portal field.
        
        Datasource: ['firewall.address.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_active_auth_scheme_references(self, client: Any) -> list[str]:
        """
        Validate active_auth_scheme references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - authentication/scheme        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     active_auth_scheme="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_active_auth_scheme_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "active_auth_scheme", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.authentication.scheme.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Active-Auth-Scheme '{value}' not found in "
                "authentication/scheme"
            )        
        return errors    
    async def validate_sso_auth_scheme_references(self, client: Any) -> list[str]:
        """
        Validate sso_auth_scheme references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - authentication/scheme        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     sso_auth_scheme="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_sso_auth_scheme_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "sso_auth_scheme", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.authentication.scheme.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Sso-Auth-Scheme '{value}' not found in "
                "authentication/scheme"
            )        
        return errors    
    async def validate_captive_portal_references(self, client: Any) -> list[str]:
        """
        Validate captive_portal references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     captive_portal="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_captive_portal_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "captive_portal", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Captive-Portal '{value}' not found in "
                "firewall/address"
            )        
        return errors    
    async def validate_captive_portal6_references(self, client: Any) -> list[str]:
        """
        Validate captive_portal6 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address6        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     captive_portal6="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_captive_portal6_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "captive_portal6", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address6.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Captive-Portal6 '{value}' not found in "
                "firewall/address6"
            )        
        return errors    
    async def validate_cert_captive_portal_references(self, client: Any) -> list[str]:
        """
        Validate cert_captive_portal references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     cert_captive_portal="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_cert_captive_portal_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "cert_captive_portal", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Cert-Captive-Portal '{value}' not found in "
                "firewall/address"
            )        
        return errors    
    async def validate_user_cert_ca_references(self, client: Any) -> list[str]:
        """
        Validate user_cert_ca references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/ca        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     user_cert_ca=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_user_cert_ca_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "user_cert_ca", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.vpn.certificate.ca.exists(value):
                found = True
            elif await client.api.cmdb.vpn.certificate.local.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"User-Cert-Ca '{value}' not found in "
                    "vpn/certificate/ca or vpn/certificate/local"
                )        
        return errors    
    async def validate_dev_range_references(self, client: Any) -> list[str]:
        """
        Validate dev_range references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     dev_range=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dev_range_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.authentication.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "dev_range", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.address.exists(value):
                found = True
            elif await client.api.cmdb.firewall.addrgrp.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Dev-Range '{value}' not found in "
                    "firewall/address or firewall/addrgrp"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_active_auth_scheme_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_sso_auth_scheme_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_captive_portal_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_captive_portal6_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_cert_captive_portal_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_user_cert_ca_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dev_range_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SettingModel",    "SettingUserCertCa",    "SettingDevRange",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.196713Z
# ============================================================================


"""FortiOS CMDB - Ldap category"""

from .query import Query

__all__ = [
    "Ldap",
    "Query",
]


class Ldap:
    """Ldap endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ldap endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.query = Query(client)
